@@ -2,14 +2,14 @@ package RedisDB;
 
 use strict;
 use warnings;
-our $VERSION = "2.09_03";
+our $VERSION = "2.13_01";
 $VERSION = eval $VERSION;
 
 use RedisDB::Error;
 use RedisDB::Parse::Redis;
 use IO::Socket::INET;
 use IO::Socket::UNIX;
-use Socket qw(MSG_DONTWAIT SO_RCVTIMEO SO_SNDTIMEO);
+use Socket qw(MSG_DONTWAIT MSG_NOSIGNAL SO_RCVTIMEO SO_SNDTIMEO);
 use POSIX qw(:errno_h);
 use Config;
 use Carp;
@@ -30,9 +30,10 @@ RedisDB - Perl extension to access redis database
 
 =head1 DESCRIPTION
 
-This module provides interface to access redis key-value store. It
-transparently handles disconnects and forks. It supports transactions,
-pipelining, and subscription mode.
+This module provides interface to access redis key-value store, it
+transparently handles disconnects and forks, supports transactions,
+pipelining, and subscription mode. Module includes XS and pure Perl
+versions of the parser.
 
 =head1 METHODS
 
@@ -89,6 +90,11 @@ Assume that all data on the server encoded in UTF-8. As result all strings will
 be converted to UTF-8 before sending to server, and all results will be decoded
 from UTF-8. See L</"UTF-8 SUPPORT">.
 
+=item connection_name
+
+After establishing connection set its name. See "CLIENT SETNAME" command
+description in the redis documentation.
+
 =item lazy
 
 by default I<new> establishes connection to the server. If this parameter is
@@ -99,7 +105,7 @@ server.
 
 this parameter allows you to specify how many attempts to (re)connect to the
 server should be made before returning error. Default value is 1, set to -1 if
-module should try to reconnect indefinetely.
+module should try to reconnect indefinitely.
 
 =item reconnect_delay_max
 
@@ -107,8 +113,6 @@ module makes a delay before each new attempt to connect. Delay increases with
 each new attempt. This parameter allows you to specify maximum delay between
 attempts to reconnect. Default value is 10.
 
-=begin comment
-
 =item on_connect_error
 
 this allows you to specify callback that will be invoked in the case of failed
@@ -116,9 +120,15 @@ connection attempt. First argument to callback is a reference to the RedisDB
 object, and second is the error description. You must not invoke any methods on
 the object, but you can change I<port> and I<host>, or I<path> attributes.
 After callback returned, module tries to establish connection again. Default
-calback confesses. This may be useful to fallback to reserve server if primary
+callback confesses. This may be useful to switch to reserve server if primary
 is down.
 
+=begin comment
+
+=item on_disconnect
+
+callback to invoke when server closed connection
+
 =end comment
 
 =back
@@ -141,6 +151,7 @@ sub new {
     $self->{reconnect_attempts}  ||= 1;
     $self->{reconnect_delay_max} ||= 10;
     $self->{on_connect_error}    ||= \&_on_connect_error;
+    $self->{on_disconnect}       ||= \&_on_disconnect;
     $self->_init_parser;
     $self->_connect unless $self->{lazy};
     return $self;
@@ -186,6 +197,28 @@ sub _on_connect_error {
     confess "Couldn't connect to the redis server at $server: $!";
 }
 
+sub _on_disconnect {
+    my ( $self, $err ) = @_;
+
+    if ($err) {
+        my $msg = "Server unexpectedly closed connection. Some data might have been lost.";
+        if ( $self->{raise_error} || $self->{_in_multi} ) {
+            confess $msg;
+        }
+        else {
+
+            # parser may be in inconsistent state, so we just replace it with a new one
+            my $parser = delete $self->{_parser};
+            $self->_init_parser;
+
+            $parser->propagate_reply( RedisDB::Error::DISCONNECTED->new($msg) );
+        }
+    }
+    else {
+        $self->{warnings} and warn "Server closed connection, reconnecting...";
+    }
+}
+
 # establish connection to the server.
 
 sub _connect {
@@ -229,7 +262,25 @@ sub _connect {
     }
 
     if ( $self->{timeout} ) {
-        my $timeout = pack( 'L!L!', $self->{timeout}, 0 );
+        my $timeout;
+        if ( $Config{osname} eq 'netbsd' and $Config{osvers} >= 6.0 and $Config{longsize} == 4 ) {
+            if ( defined $Config{use64bitint} ) {
+                $timeout = pack( 'QL', $self->{timeout}, 0 );
+            }
+            else {
+                $timeout = pack(
+                    'LLL',
+                    (
+                        $Config{byteorder} eq '1234'
+                        ? ( $self->{timeout}, 0, 0 )
+                        : ( 0, $self->{timeout}, 0 )
+                    )
+                );
+            }
+        }
+        else {
+            $timeout = pack( 'L!L!', $self->{timeout}, 0 );
+        }
         try {
             defined $self->{_socket}->sockopt( SO_RCVTIMEO, $timeout )
               or die "Can't set timeout: $!";
@@ -258,6 +309,11 @@ sub _connect {
         );
     }
 
+    # connection name
+    if ( $self->{connection_name} ) {
+        $self->send_command( qw(CLIENT SETNAME), $self->{connection_name}, IGNORE_REPLY() );
+    }
+
     # select database
     if ( $self->{database} ) {
         $self->send_command( "SELECT", $self->{database}, IGNORE_REPLY() );
@@ -301,15 +357,18 @@ sub _recv_data_nb {
             $self->{_parser}->add($buf);
         }
         else {
+            delete $self->{_socket};
 
-            # if there are some replies lost
-            confess "Server closed connection. Some data was lost."
-              if $self->{_parser}->callbacks
-              or $self->{_in_multi};
+            if($self->{_parser}->callbacks or $self->{_in_multi}) {
+                # there are some replies lost
+                $self->{on_disconnect}->($self, 1);
+            }
+            else {
+                # clean disconnect, try to reconnect
+                $self->{on_disconnect}->($self, 0);
+            }
 
-            # clean disconnect, try to reconnect
-            $self->{warnings} and warn "Disconnected, trying to reconnect";
-            $self->_connect;
+            $self->_connect unless $self->{_socket};
             last;
         }
     }
@@ -328,7 +387,7 @@ sub _queue {
 =head2 $self->send_command($command[, @arguments][, \&callback])
 
 send a command to the server. Returns true if the command was successfully
-sent, or dies if an error occured. Note, that it does not return reply from
+sent, or dies if an error occurred. Note, that it does not return reply from
 the server, you should retrieve it using the I<get_reply> method, or if I<callback>
 has been specified, it will be invoked upon receiving the reply with two
 arguments: the RedisDB object, and the reply from the server.  If the server
@@ -344,6 +403,8 @@ I<reply_ready>, I<get_reply>, or I<get_all_replies>.
 
 =cut
 
+my $NOSIGNAL = try { MSG_NOSIGNAL } || 0;
+
 sub send_command {
     my $self = shift;
 
@@ -368,7 +429,7 @@ sub send_command {
     }
 
     # if SELECT has been successful, we should update database
-    if ( $command eq 'SELECT' ) {
+    elsif ( $command eq 'SELECT' ) {
         my $cb    = $callback;
         my $dbnum = $_[0];
         $callback = sub {
@@ -377,6 +438,11 @@ sub send_command {
         };
     }
 
+    # if CLIENT SETNAME we should remember the name
+    elsif ( $command eq 'CLIENT' && uc $_[0] eq 'SETNAME' ) {
+        $self->{connection_name} = $_[1];
+    }
+
     $self->_connect unless $self->{_socket} and $self->{_pid} == $$;
 
     # Here we are reading received data and parsing it,
@@ -384,7 +450,11 @@ sub send_command {
     $self->_recv_data_nb;
 
     my $request = $self->{_parser}->build_request( $command, @_ );
-    defined $self->{_socket}->send($request) or confess "Can't send request to server: $!";
+    {
+        local $SIG{PIPE} = 'IGNORE' unless $NOSIGNAL;
+        defined $self->{_socket}->send( $request, $NOSIGNAL )
+          or confess "Can't send request to server: $!";
+    }
     $self->{_parser}->add_callback($callback);
     return 1;
 }
@@ -519,8 +589,8 @@ sub get_reply {
         }
         else {
 
-            # disconnected
-            confess "Server unexpectedly closed connection before sending full reply";
+            # disconnected, should die unless raise_error is unset
+            $self->{on_disconnect}->($self, 1);
         }
     }
 
@@ -603,16 +673,18 @@ sub version {
 
 # don't forget to update POD
 my @commands = qw(
-  append	auth	bgrewriteaof	bgsave	blpop	brpop   brpoplpush	config	config_get
-  config_set	config_resetstat	dbsize	debug_object	debug_segfault
+  append	auth	bgrewriteaof	bgsave	bitcount	bitop
+  blpop	brpop   brpoplpush	client	client_kill	client_getname	client_setname
+  config	config_get	config_set	config_resetstat
+  dbsize	debug_object	debug_segfault
   decr	decrby	del	dump	echo	eval    evalsha exists	expire	expireat	flushall
   flushdb	get	getbit	getrange	getset	hdel	hexists	hget	hgetall
   hincrby	hincrbyfloat	hkeys	hlen	hmget	hmset	hset	hsetnx	hvals	incr	incrby
   incrbyfloat	keys	lastsave	lindex	linsert	llen	lpop	lpush	lpushx
   lrange	lrem	lset	ltrim	mget	migrate	move	mset	msetnx	object	object_refcount
-  object_encoding	object_idletime	persist	pexpire	pexpireat	psetex	pttl	ping
+  object_encoding	object_idletime	persist	pexpire	pexpireat	ping	psetex	pttl
   publish	quit	randomkey	rename	renamenx	restore	rpop	rpoplpush
-  rpush	rpushx	sadd	save	scard	script_exists   script_flush    script_kill
+  rpush	rpushx	sadd	save	scard	script	script_exists   script_flush    script_kill
   script_load   sdiff	sdiffstore	select	set
   setbit	setex	setnx	setrange	sinter	sinterstore
   sismember	slaveof	slowlog smembers	smove	sort	spop	srandmember
@@ -623,6 +695,10 @@ my @commands = qw(
   zscore	zunionstore
 );
 
+sub _simple_commands {
+    return @commands;
+}
+
 =head1 WRAPPER METHODS
 
 Instead of using I<execute> and I<send_command> methods directly, it may be
@@ -643,22 +719,23 @@ I<execute>, waits for the reply from the server, and returns it. E.g.:
     $redis->send_command("get", $key, sub { $val = $_[1] });
 
 The following wrapper methods are defined: append, auth, bgrewriteaof, bgsave,
-blpop, brpop,   brpoplpush, config, config_get, config_set, config_resetstat,
-dbsize, debug_object, debug_segfault, decr, decrby, del, dump, echo, eval,
-evalsha, exists, expire, expireat, flushall, flushdb, get, getbit, getrange,
-getset, hdel, hexists, hget, hgetall, hincrby, hincrbyfloat, hkeys, hlen,
-hmget, hmset, hset, hsetnx, hvals, incr, incrby, incrbyfloat, keys, lastsave,
-lindex, linsert, llen, lpop, lpush, lpushx, lrange, lrem, lset, ltrim, mget,
-migrate, move, mset, msetnx, object, object_refcount, object_encoding,
-object_idletime, persist, pexpire, pexpireat, psetex, pttl, ping, publish,
-quit, randomkey, rename, renamenx, restore, rpop, rpoplpush, rpush, rpushx,
-sadd, save, scard, script_exists,   script_flush, script_kill, script_load,
-sdiff, sdiffstore, select, set, setbit, setex, setnx, setrange, sinter,
-sinterstore, sismember, slaveof, slowlog, smembers, smove, sort, spop,
-srandmember, srem, strlen, sunion, sunionstore, sync, time,    ttl, type,
-unwatch, watch, zadd, zcard, zcount, zincrby, zinterstore, zrange,
-zrangebyscore, zrank, zrem, zremrangebyrank,   zremrangebyscore, zrevrange,
-zrevrangebyscore, zrevrank, zscore, zunionstore.
+bitcount, bitop, blpop, brpop, brpoplpush, client, client_kill, client_getname,
+client_setname, config, config_get, config_set, config_resetstat, dbsize,
+debug_object, debug_segfault, decr, decrby, del, dump, echo, eval, evalsha,
+exists, expire, expireat, flushall, flushdb, get, getbit, getrange, getset,
+hdel, hexists, hget, hgetall, hincrby, hincrbyfloat, hkeys, hlen, hmget, hmset,
+hset, hsetnx, hvals, incr, incrby, incrbyfloat, keys, lastsave, lindex,
+linsert, llen, lpop, lpush, lpushx, lrange, lrem, lset, ltrim, mget, migrate,
+move, mset, msetnx, object, object_refcount, object_encoding, object_idletime,
+persist, pexpire, pexpireat, ping, psetex, pttl, publish, quit, randomkey,
+rename, renamenx, restore, rpop, rpoplpush, rpush, rpushx, sadd, save, scard,
+script, script_exists, script_flush, script_kill, script_load, sdiff,
+sdiffstore, select, set, setbit, setex, setnx, setrange, sinter, sinterstore,
+sismember, slaveof, slowlog, smembers, smove, sort, spop, srandmember, srem,
+strlen, sunion, sunionstore, sync, time, ttl, type, unwatch, watch, zadd,
+zcard, zcount, zincrby, zinterstore, zrange, zrangebyscore, zrank, zrem,
+zremrangebyrank, zremrangebyscore, zrevrange, zrevrangebyscore, zrevrank,
+zscore, zunionstore.
 
 See description of all commands in redis documentation at
 L<http://redis.io/commands>.
@@ -681,11 +758,11 @@ for my $command (@commands) {
 
 =pod
 
-The following commands implement some additional postprocessing of results:
+The following commands implement some additional postprocessing of the results:
 
 =cut
 
-=head2 $self->info
+=head2 $self->info([$callback])
 
 return information and statistics about the server. Redis-server returns
 information in form of I<field:value>, the I<info> method parses result and
@@ -695,12 +772,62 @@ returns it as a hash reference.
 
 sub info {
     my $self = shift;
+    my $orig = $_[-1];
+    if ( $orig && ref $orig eq 'CODE' ) {
+        my $cb = sub {
+            my ( $redis, $info ) = @_;
+            $orig->( $redis, _parse_info($info) );
+        };
+        return $self->send_command( 'INFO', $cb );
+    }
+    else {
+        my $info = $self->execute('INFO');
+        return _parse_info($info);
+    }
+}
 
-    my $info = $self->execute('INFO');
+sub _parse_info {
+    my $info = shift;
+    return $info if !$info || ref $info;
     my %info = map { /^([^:]+):(.*)$/ } split /\r\n/, $info;
     return \%info;
 }
 
+=head2 $self->client_list([$callback])
+
+return list of clients connected to the server. This method parses server
+output and returns result as reference to array of hashes.
+
+=cut
+
+sub client_list {
+    my $self = shift;
+    my $orig = $_[-1];
+    if ( $orig && ref $orig eq 'CODE' ) {
+        my $cb = sub {
+            my ( $redis, $list ) = @_;
+            $orig->( $redis, _parse_client_list($list) );
+        };
+        return $self->send_command( qw(CLIENT LIST), $cb );
+    }
+    else {
+        my $list = $self->execute(qw(CLIENT LIST));
+        return _parse_client_list($list);
+    }
+}
+
+sub _parse_client_list {
+    my $list = shift;
+    return $list if !$list || ref $list;
+    my @clients = split /\015?\012/, $list;
+    my $res = [];
+    for (@clients) {
+        my %cli = map { /^([^=]+)=(.*)$/ ? ( $1, $2 ) : () } split / /;
+        push @$res, \%cli;
+    }
+    return $res;
+}
+
 =head2 $self->shutdown
 
 Shuts the redis server down. Returns undef, as the server doesn't send the
@@ -740,7 +867,7 @@ It may happen as a result of a network error or invalid data encoding. Also
 module will throw an exception of the L<RedisDB::Error> type if the server
 returned an error reply and I<raise_error> parameter is set to true in the
 constructor (which is by default). After throwing an exception other than the
-L<RedisDB::Error> the RedisDB object will be left in inconsistant state. If you
+L<RedisDB::Error> the RedisDB object will be left in inconsistent state. If you
 want to continue using the object after getting such an exception, you should
 invoke the L</reset_connection> method on it. This will drop current connection
 and all outstanding requests, so the object will return to the same state it
@@ -781,7 +908,7 @@ will never be invoked:
     $redis->send_command( "ping", sub { $pong = $_[1] } );
     sleep 1 while not $pong;    # this will never return
 
-Therefore you need perriodically trigger check for the replies. The check is
+Therefore you need periodically trigger check for the replies. The check is
 triggered when you call the following methods: I<send_command>, I<reply_ready>,
 I<get_reply>, I<get_all_replies>.  Calling wrapper method, like
 C<< $redis->get('key') >>, will also trigger check as internally wrapper methods
@@ -872,7 +999,7 @@ You can publish messages into the channels using the I<publish> method. This
 method should be called when you in the normal mode, and can't be used while
 you're in the subscription mode.
 
-Following methods can be used in subscribtion mode:
+Following methods can be used in subscription mode:
 
 =cut
 
@@ -908,14 +1035,14 @@ same as subscribe, but you specify patterns for channels' names.
 =back
 
 All parameters are optional, but you must subscribe at least to one channel. Also
-if default_callback is not specified, you have to explicitely specify a callback
+if default_callback is not specified, you have to explicitly specify a callback
 for every channel you are going to subscribe.
 
 =cut
 
 sub subscription_loop {
     my ( $self, %args ) = @_;
-    croak "Already in subscription loop" if $self->{_subscribtion_loop};
+    croak "Already in subscription loop" if $self->{_subscription_loop};
     croak "You can't start subscription loop while in pipelining mode."
       if $self->{_parser}->callbacks
       or @{ $self->{_replies} };
@@ -1097,14 +1224,14 @@ sub psubscribed {
 Transactions allow you to execute a sequence of commands in a single step. In
 order to start a transaction you should use the I<multi> method.  After you
 have entered a transaction all the commands you issue are queued, but not
-executed till you call the I<exec> method. Tipically these commands return
+executed till you call the I<exec> method. Typically these commands return
 string "QUEUED" as a result, but if there is an error in e.g. number of
 arguments, they may croak. When you call exec, all the queued commands will be
 executed and exec will return a list of results for every command in the
 transaction. If instead of I<exec> you call I<discard>, all scheduled commands
 will be canceled.
 
-You can set some keys as watched. If any whatched key has been changed by
+You can set some keys as watched. If any watched key has been changed by
 another client before you called exec, the transaction will be discarded and
 exec will return false value.
 
@@ -1170,9 +1297,9 @@ I was in need of the client for redis database. L<AnyEvent::Redis> didn't suite
 me as it requires an event loop, and it didn't fit into the existing code. The
 problem with L<Redis> is that it didn't (at the time I started this) reconnect
 to the server if connection was closed after timeout or as result of the server
-restart, and it does not support pipelining. After analizing what I need to
+restart, and it does not support pipelining. After analyzing what I need to
 change in L<Redis> in order to get all I want, I decided that it will be
-simplier to write the new module from scratch. This also solves the problem of
+simpler to write the new module from scratch. This also solves the problem of
 backward compatibility.
 
 =head1 BUGS
@@ -1199,7 +1326,7 @@ Pavel Shaydo, C<< <zwon at cpan.org> >>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2011, 2012 Pavel Shaydo.
+Copyright 2011-2013 Pavel Shaydo.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
