@@ -2,7 +2,7 @@ package RedisDB::Parse::Redis;
 
 use strict;
 use warnings;
-our $VERSION = "2.09_03";
+our $VERSION = "2.13_01";
 $VERSION = eval $VERSION;
 
 use Try::Tiny;
@@ -82,6 +82,11 @@ Used by subscription loop.
 
 Returns true if there are callbacks in queue
 
+=head2 $self->propagate_reply($reply)
+
+Invoke every callback from queue and the default callback with the given
+I<$reply>.
+
 =head2 $self->add($data)
 
 Process new data received from the server. For every reply found callback from
@@ -104,7 +109,7 @@ Pavel Shaydo, C<< <zwon at cpan.org> >>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2011, 2012 Pavel Shaydo.
+Copyright 2011-2013 Pavel Shaydo.
 
 This program is free software; you can redistribute it and/or modify it
 under the terms of either: the GNU General Public License as published
